# -*- coding: utf-8 -*-
#
##########################################################################
# eoleflask.loader -- loading flask applications
# Copyright © 2013 Pôle de compétences EOLE <eole@ac-dijon.fr>
#
# License CeCILL:
#  * in french: http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
#  * in english http://www.cecill.info/licences/Licence_CeCILL_V2-en.html
##########################################################################

"""Load flask applications

This module define the function :func:`get_apps_list` to build a dictionary
of ``mount_point``:``module`` for use by :mode:`werkzeug.wsgi`
dispatcher.

"""

import logging

from os import walk
from os.path import join
import json

from pyeole.log import make_logger

APPS_ENABLE_DIR = "/etc/eole/flask/enabled"
"""Directory of enabled eoleflask applications

"""

_LOG_DIR = '/var/log/eoleflask'
"""Directory to store log files.

"""

_STATIC_BASE = '/usr/share/eole/flask'
"""Base directory where static files are stored per application
"""

log = make_logger('eoleflask', logfile=join(_LOG_DIR, 'eoleflask.log'),
                  loglevel='info')

class NoApplicationError(StandardError):
    """Unable to load some applications

    """
    pass


#APPS_ENABLE_DIR="/etc/eole/flask/available"

def load_app(configfile=None):
    """Load a flask application and its configuration
     in global namespace

    :param configfile: configuration file
    :type module: `str`
    :return: loaded flask app object
    :rtype: `flask.app`

    """
    if configfile is not None:
        with open(join(APPS_ENABLE_DIR,configfile), 'r') as f:
            # Load the config for the current app
            try:
                config = json.loads(f.read().decode('utf-8'))
            except StandardError, err:
                log.error("Unable to load json config file %s" % f.name,
                          exc_info=True)
                return False

	if not config.has_key('APPNAME'):
            log.error('Missing "APPNAME" key in config')
            return False

	module = config['APPNAME']
	# Try to import the current app
        try:
            m = __import__(module)
        except StandardError, err:
            log.error("Unable to load module '%s': %s" % (module, err),
                      exc_info=True)
            return False

        # Where to look for resources
        if 'INSTANCE_PATH' not in config:
            config['INSTANCE_PATH'] = join(_STATIC_BASE,
                                           config['MOUNT_POINT'].lstrip('/'),
                                           'resources')

        # Prefix of URL for static files
        if 'STATIC_URL_PATH' not in config:
            config['STATIC_URL_PATH'] = '/static'

        # Where to lookup static files
        if 'STATIC_FOLDER' not in config:
            config['STATIC_FOLDER'] = join(_STATIC_BASE,
                                           config['MOUNT_POINT'].lstrip('/'),
                                           'static')
        elif config['STATIC_FOLDER'] != '/static':
            m.app.add_url_rule(config['STATIC_FOLDER'] + '/<path:filename>',
                               endpoint='static',
                               view_func=m.app.send_static_file)

        # Where to lookup templates
        if 'TEMPLATE_FOLDER' not in config:
            config['TEMPLATE_FOLDER'] = join(_STATIC_BASE,
                                             config['MOUNT_POINT'].lstrip('/'),
                                             'templates')

        if 'SECRET_KEY' in config:
             if isinstance(config['SECRET_KEY'], unicode):
                 config['SECRET_KEY'] = config['SECRET_KEY'].encode('utf8')

        #Needed to avoid propagation of exception
        config['DEBUG'] = False
        config['PROPAGATE_EXCEPTIONS'] = False

        m.app.root_path = config['INSTANCE_PATH']

        m.app.config.root_path = APPS_ENABLE_DIR

        m.app.instance_path = config['INSTANCE_PATH']

        m.app.static_url_path = config['STATIC_URL_PATH']

        m.app.static_folder = config['STATIC_FOLDER']

        m.app.template_folder = config['TEMPLATE_FOLDER']

        m.app.config.update(config)

        # Add logger handler
        logfile = join(_LOG_DIR, '{0}.log'.format(config['APPNAME']))
        file_handler = logging.FileHandler(filename=logfile)
        m.app.logger.addHandler(file_handler)
        if 'LOG_LEVEL' in config.keys():
            # Default log level is WARNING
            m.app.logger.setLevel(getattr(logging, config['LOG_LEVEL'].upper()))

        # return the app
        return m.app

def get_apps_list():
    """Load all flask applications and configurations

    :return: dict of mount points and associated apps
    :rtype: dict
    :raise NoApplicationError: if no application is loaded.

    """
    apps = {}
    for root, dirs, filenames in walk(APPS_ENABLE_DIR):
        for name in filenames:
            if not name.endswith('.conf'):
                continue
            app = load_app(name)
            if app:
                # populate the array of apps
                apps[app.config['MOUNT_POINT']] = app

    if len(apps) == 0:
        raise NoApplicationError('No application loaded')

    return apps

